function temperatureCA(niter)
%function temperatureCA(niter)
%
% This function implements a cellular automaton model that uses the mean
% value property to approximate the solution to the Laplace equation
% in order to calculate the steady-state temperature at
% the center of a homogeneous rectangular two-dimensional object with one
% side fixed at 100 degrees and the other three sides fixed at 0 degrees.
% The rectangle is twice as long as it is wide.  It is discretized into 201
% columns and 101 rows, for a total of 20,301 cells.  At each iteration,
% the value of each cell is calculated to be the mean of the values of the
% cells in its Moore neighborhood (eight surrounding cells), where the
% corner cells of the neighborhood (Northwest, Northeast, Southwest,
% and Southeast) are given 1/4 the weighting of the four cells immediately
% adjacent to the cell in question (North, East, South and West).  All
% values are rounded to the nearest ten-thousandth (10e-4) at the end of
% each iteration.  A history of the value of the central cell is kept
% during the simulation and plotted upon completion, along with a
% color-coded plot of the temperature across the rectangle.
%
% If the argument niter is provided, then the cellular automaton simulation
% will proceed for niter iterations.  Otherwise, it will proceed until the
% value of the central cell (rounded to the nearest ten-thousandth) has not
% changed upon two consecutive iterations.
%
% (C) Dylan Hirsch-Shell, Nov 2006

nrows = 101;
ncolumns = 201;

temperature = zeros(nrows,ncolumns);  % 101 x 201 rectangle of cells

temperature(1,1:ncolumns)=100;

centralcellvaluehist = [];

if nargin>0
    for iter = 0:niter-1
        [temperature,centralcellvaluehist] = updatetemp(temperature,centralcellvaluehist);
    end
else
    iter = 1;
    while (iter<400) || (abs(centralcellvaluehist(end)-centralcellvaluehist(end-1))>0.0000001)
        [temperature,centralcellvaluehist] = updatetemp(temperature,centralcellvaluehist);
        iter = iter + 1;
    end
    display(iter)
    display(centralcellvaluehist(end))
    figure; plot(centralcellvaluehist)
end

figure; imagesc(temperature)
shading flat
axis image
colorbar

function [temperature,centralcellvaluehist] = updatetemp(temperature,centralcellvaluehist)
[nrows,ncolumns]=size(temperature);
update = temperature;
for row = 2:(nrows-1)
    for column = 2:(ncolumns-1)
        NW = temperature(row-1,column-1);
        N = temperature(row-1,column);
        NE = temperature(row-1,column+1);
        W = temperature(row,column-1);
        E = temperature(row,column+1);
        SW = temperature(row+1,column-1);
        S = temperature(row+1,column);
        SE = temperature(row+1,column+1);
        update(row,column) = (N+W+E+S)/5 + (NW+NE+SW+SE)/20;
    end
end
temperature = update;

centralcellvaluehist = [centralcellvaluehist temperature(51,101)];
temperature = round(temperature.*10000)/10000;
